{ Component "CompCtrl V2.0" is a non-visual component witch enables moving and
  resizing visible and enabled controls on all application-forms at runtime.
  OnControlChange signals the component under mousecursor.

  Udo Juerss
  57078 Siegen, Germany
  April 1999

  e-mail: ujhs@aol.com

  Only four properties:

  EnableMove: Bool; enables moving the control (under mousecursor) by pressing
                    shift-key and moving the mouse.

  EnableSize: Bool; enables sizing the control (under mousecursor) by pressing
                    ctrl-key and moving the mouse.

  EnableFront: Bool; enables bringing to front the control (under mousecursor)
                     by pressing F12.

  OnChange: TControlChange;  enables to retrieve the component on witch the
                             mouse moves. If none, it returns the Form.
                             (Nice option to feed an own debugger window with
                              component data at runtime)

  Hint: You can disable handling of a single control by setting its property
        "Tag" to a nonzero value.
        Its easy to change control-keys for moving, sizing or fronting by
        assigning other VK_xxx codes to VK_MOVE, VK_SIZE and VK_FRONT.
}

unit
  CompCtrl;

interface
{------------------------------------------------------------------------------}

uses
  Windows,Messages,Classes,Forms,Dialogs,Controls,StdCtrls;
{------------------------------------------------------------------------------}

const
  VK_MOVE = VK_SHIFT;
  VK_SIZE = VK_CONTROL;
  VK_FRONT = VK_F12;
{------------------------------------------------------------------------------}

type
  TControlChange = procedure(Sender: TObject; Ctrl: TControl) of object;

  TCompCtrl = class(TComponent)
  private
    FEnableMove: Bool;
    FEnableSize: Bool;
    FEnableFront: Bool;
    FOnChange: TControlChange;
  protected
    procedure SetBool(Index: Integer; Value: Bool);
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    property EnableMove: Bool index 0 read FEnableMove write SetBool;
    property EnableSize: Bool index 1 read FEnableSize write SetBool;
    property EnableFront: Bool index 2 read FEnableFront write SetBool;
    property OnChange: TControlChange read FOnChange write FOnChange;
  end;
{------------------------------------------------------------------------------}

procedure Register;

implementation
{------------------------------------------------------------------------------}

var
  ParentForm: TCustomForm;
  MouseHandle: HHook;
  KeyHandle: HHook;
  MoveEnable: Bool;
  SizeEnable: Bool;
  FrontEnable: Bool;
  Moving: Bool;
  Sizing: Bool;
  Fronting: Bool;
  FindBusy: Bool;
  Cursor: TCursor;
  ActPt: TPoint;
  LastPt: TPoint;
  FControl: TControl;
  VControl: TControl;
  ChildVControl: TControl;
  CCtrl: TCompCtrl;
{------------------------------------------------------------------------------}

function MouseHookCallBack(Code: Integer; Msg: WPARAM; Mouse: LPARAM): LRESULT; stdcall;
begin
  if Code >= 0 then
  begin
    ParentForm:=Screen.ActiveCustomForm;
    if Assigned(ParentForm) then
    begin
      if not FindBusy and not (Moving or Sizing or Fronting) then
      begin
        FindBusy:=True;
        if (Msg = WM_MOUSEMOVE) or (Msg = WM_NCMOUSEMOVE) then
        begin
          ActPt:=PMouseHookStruct(Mouse)^.Pt;
          ActPt:=ParentForm.ScreenToClient(ActPt);
        end
        else
        begin
          GetCursorPos(ActPt);
          ActPt:=ParentForm.ScreenToClient(ActPt);
        end;
        VControl:=ParentForm.ControlAtPos(ActPt,False);
        if (VControl = nil) then
        begin
          ActPt:=ParentForm.ClientToScreen(ActPt);
          VControl:=FindVCLWindow(ActPt);
          if (VControl <> nil) and (VControl <> ParentForm) then
          begin
            ActPt:=VControl.ScreenToClient(ActPt);
            ChildVControl:=TWinControl(VControl).ControlAtPos(ActPt,False);
            if (ChildVControl <> nil) then VControl:=ChildVControl;
          end;
        end;
        if (FControl <> VControl) then
        begin
          if Assigned(TCompCtrl(CCtrl).FOnChange) then
            TCompCtrl(CCtrl).FOnChange(nil,VControl);
          FControl:=VControl;
        end;
        FindBusy:=False;
      end;
    end;

    if Assigned(FControl) and (FControl.Tag = 0) and
      (Msg = WM_MOUSEMOVE) and (Moving or Sizing) then
    begin
      ActPt:=PMouseHookStruct(Mouse)^.Pt;
      if Moving then
      begin
        FControl.Left:=FControl.Left + ActPt.X - LastPt.X;
        FControl.Top:=FControl.Top + ActPt.Y - LastPt.Y;
      end
      else if Sizing then
      begin
        FControl.Width:=FControl.Width + ActPt.X - LastPt.X;
        FControl.Height:=FControl.Height + ActPt.Y - LastPt.Y;
      end;
      LastPt:=ActPt;
    end;
  end;
  Result:=CallNextHookEx(MouseHandle,Code,Msg,Mouse);
end;
{------------------------------------------------------------------------------}

function KeyHookCallBack(Code: Integer; VK: WPARAM; Key: LPARAM): LRESULT; stdcall;
begin
  if (Code >= 0) and Assigned(ParentForm) then
  begin
    if VK = VK_MOVE then
    begin
      if (HiWord(Key) and KF_UP = 0) and MoveEnable and not (Moving or Sizing) then
      begin
        Moving:=True;
        Cursor:=Screen.Cursor;
        Screen.Cursor:=crSizeAll;
        GetCursorPos(LastPt);
      end;

      if (HiWord(Key) and KF_UP = KF_UP) and Moving then
      begin
        Moving:=False;
        Screen.Cursor:=Cursor;
      end;
    end;

    if VK = VK_SIZE then
    begin
      if (HiWord(Key) and KF_UP = 0) and SizeEnable and not (Moving or Sizing) then
      begin
        Sizing:=True;
        Cursor:=Screen.Cursor;
        Screen.Cursor:=crSizeAll;
        GetCursorPos(LastPt);
      end;

      if (HiWord(Key) and KF_UP = KF_UP) and Sizing then
      begin
        Sizing:=False;
        Screen.Cursor:=Cursor;
      end;
    end;

    if VK = VK_FRONT then
    begin
      if (HiWord(Key) and KF_UP = 0) and FrontEnable then
        FControl.BringToFront;
    end;
  end;

  Result:=CallNextHookEx(KeyHandle,Code,VK,Key);
end;
{------------------------------------------------------------------------------}

constructor TCompCtrl.Create(AOwner: TComponent);
var
  AppName: array[0..127] of Char;
  HModule: THandle;
begin
  inherited;
  FEnableMove:=True;
  FEnableSize:=True;
  FEnableFront:=True;
  MoveEnable:=True;
  SizeEnable:=True;
  FrontEnable:=True;
  CCtrl:=Self;
  MouseHandle:=0;
  KeyHandle:=0;

  if not (csDesigning in ComponentState) then
  begin
    if GetModuleFileName(HInstance,AppName,Pred(SizeOf(AppName))) > 0 then
    begin
      HModule:=GetModuleHandle(AppName);
      if HModule > 0 then
      begin
        MouseHandle:=SetWindowsHookEx(WH_MOUSE,MouseHookCallBack,HModule,GetCurrentThreadID);
        KeyHandle:=SetWindowsHookEx(WH_KEYBOARD,KeyHookCallBack,HModule,GetCurrentThreadID);
      end;
    end;
    if (MouseHandle = 0) or (KeyHandle = 0) then ShowMessage('Threadhook failed!');
  end;
end;
{------------------------------------------------------------------------------}

destructor TCompCtrl.Destroy;
begin
  if MouseHandle <> 0 then UnhookWindowsHookEx(MouseHandle);
  if KeyHandle <> 0 then UnhookWindowsHookEx(KeyHandle);
  inherited;
end;
{------------------------------------------------------------------------------}

procedure TCompCtrl.SetBool(Index: Integer; Value: Bool);
begin
  case Index of
    0: begin
         FEnableMove:=Value;
         MoveEnable:=Value;
       end;
    1: begin
         FEnableSize:=Value;
         SizeEnable:=Value;
       end;
    2: begin
         FEnableFront:=Value;
         FrontEnable:=Value;
       end;
  end;
end;
{------------------------------------------------------------------------------}

procedure Register;
begin
  RegisterComponents('Udo',[TCompCtrl]);
end;
{------------------------------------------------------------------------------}

initialization
end.

